#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
import json
import pprint
import datetime

import flask

import coshed_flask
from coshed_flask.tools import generate_expires_header

API_VERSION = os.environ.get("API_VERSION")
API_VERSION_FALLBACK = "0.0.42"

if API_VERSION is None:
    try:
        API_VERSION = coshed_flask.__version__
    except Exception:
        API_VERSION = API_VERSION_FALLBACK

NAVIGATION_ITEMS = [
    ("/", "Home"),
    ("/doc", "Documentation"),
]


class AppResponse(dict):
    """
    Container class for flask app responses.
    """

    def __init__(self, *args, **kwargs):
        self.drop_dev = kwargs.get("drop_dev")

        try:
            del kwargs["drop_dev"]
        except Exception:
            pass

        dict.__init__(self, *args, **kwargs)

        if "_dev" not in self:
            self["_dev"] = dict()

        try:
            self["version"] = API_VERSION
        except NameError:
            self["version"] = API_VERSION_FALLBACK

        self["python_version"] = sys.version_info.major
        self["_now"] = datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S")

        if "navigation" not in list(self.keys()):
            nav_items = list(self.navigation)
            if nav_items:
                self["navigation"] = nav_items

    @property
    def navigation(self):
        for data in NAVIGATION_ITEMS:
            try:
                (i_url, i_label, whitelist_key) = data
            except ValueError:
                whitelist_key = None
                (i_url, i_label) = data

            if whitelist_key and whitelist_key not in self.get(
                "whitelist", []
            ):
                continue
            yield i_url, i_label

    def flask_obj(
        self,
        status_code=200,
        drop_dev=None,
        with_version=True,
        expires=None,
        headers=None,
        not_to_be_exposed=None,
    ):
        """
        Generate a :py:class:`flask.Response` object for current application
        response object.

        Args:
            status_code (int): HTTP status code, default 200
            drop_dev (bool): Drop development information
            with_version (bool): include version information
            expires: expiration specification
            headers (dict): headers
            not_to_be_exposed (iterable, optional): key/value pairs not to be exposed

        Returns:
            flask.Response: HTTP response object
        """
        del_keys = []

        if drop_dev is None:
            drop_dev = self.drop_dev

        if drop_dev:
            del_keys.append("_dev")

        if with_version is False:
            del_keys.append("version")

        if not_to_be_exposed:
            for ntbe in not_to_be_exposed:
                del_keys.append(ntbe)

        for del_key in del_keys:
            try:
                del self[del_key]
            except KeyError:
                pass

        if headers is None:
            headers = dict()

        headers["Content-Type"] = "application/json"

        if expires is not None:
            headers.update(generate_expires_header(expires))

        try:
            body = json.dumps(self, indent=2, sort_keys=True)
        except TypeError:
            body = pprint.pformat(self)
            headers["Content-Type"] = "text/plain"
            status_code = 500

        return flask.Response(
            status=status_code, headers=headers, response=body
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

import hashlib

from werkzeug.security import check_password_hash
from werkzeug.security import generate_password_hash


def username_identifier(username):
    """
    Generate an identifier for given username

    Args:
        username (str): username

    Returns:
        str: user identifier
    """
    hasher = hashlib.sha1()
    hasher.update(username.encode("utf-8"))

    return f"u_{hasher.hexdigest()}".upper()


def check_password_hash_env(username, password):
    """
    Check if given credentials match user identifier/password hash pair
    in environment variables

    Args:
        username (str): username
        password (str): password

    Returns:
        bool: ``True`` if credentials match
    """
    user_id = username_identifier(username)
    challenge = os.environ.get(user_id)

    if not challenge:
        return False

    return check_password_hash(challenge, password)


def hash_tuple(username, password, method=None):
    """
    Generate a tuple containing user identifier and password hash for
    given credentials

    Args:
        username (str): username
        password (str): password
        method (str, optional): Password hashing method. \
            Defaults to ``pbkdf2:sha256:100000``.

    Returns:
        tuple: user identifier and password hash

    >>> user = "test 123"
    >>> password = "$wordfishöäü"
    >>> k, v = hash_tuple(user, password)
    >>> os.environ[k] = v
    >>> check_password_hash_env(user, password)
    True
    >>> check_password_hash_env(user, "wrong")
    False
    >>> check_password_hash_env("no-user", "wrong")
    False
    """
    if method is None:
        method = "pbkdf2:sha256:100000"

    password_hash = generate_password_hash(password, method)
    user_id = username_identifier(username)

    return (user_id, password_hash)


if __name__ == "__main__":
    import doctest

    (FAILED, SUCCEEDED) = doctest.testmod()
    print("[doctest] SUCCEEDED/FAILED: {:d}/{:d}".format(SUCCEEDED, FAILED))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import tempfile
import uuid
import logging
from logging.handlers import RotatingFileHandler
import datetime
import time
from wsgiref.handlers import format_date_time

from flask import request
from flask_cors import CORS
from flask_compress import Compress
import six


ROTATING_LOG_BACKUP_COUNT = 1
ROTATING_LOG_MAX_BYTES = 20000

#: custom log formatter
formatter = logging.Formatter(
    fmt="%(asctime)s %(name)-16s %(levelname)-8s %(funcName)-15s "
    "(#%(lineno)04d): %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

WANT_DEV = ("i-like-my-sugar-with", "coffee-and-cream")


def drop_dev(want_dev=None):
    if want_dev is None:
        want_dev = WANT_DEV

    try:
        (arg_name, expectation) = want_dev
        return not request.args.get(arg_name) == expectation
    except Exception:
        pass

    return True


def rotating_app_log(flask_app_instance, app_name=None, **kwargs):
    """
    Set up a rotating log file for flask instance.

    Args:
        flask_app_instance (flask.Flask): flask instance
        app_name (str): app name

    Keyword Args:
        maxBytes (int): maximum bytes threshold for rollover
        backupCount (int): maximum number of rotations to keep

    Returns:
        str: log path
    """
    if app_name is None:
        app_name = uuid.uuid4().hex

    log_trunk = "{:s}-webapp.log".format(app_name)
    log_filename = os.path.join(tempfile.gettempdir(), log_trunk)
    handler = RotatingFileHandler(
        log_filename,
        maxBytes=kwargs.get("maxBytes", ROTATING_LOG_MAX_BYTES),
        backupCount=kwargs.get("backupCount", ROTATING_LOG_BACKUP_COUNT),
    )
    handler.setLevel(logging.DEBUG)
    handler.setFormatter(formatter)
    flask_app_instance.logger.addHandler(handler)
    flask_app_instance.logger.setLevel(logging.INFO)
    logging.getLogger("iotta.boarding_queue").addHandler(handler)

    return log_filename


def wolfication(flask_app_instance, **kwargs):
    """
    Set up a flask instance with CORS and compression enabled and set some
    default configuration values.

    Args:
        flask_app_instance (flask.Flask): flask instance

    Keyword Args:
        jinja_filters (dict, optional): jinja filters to be installed
        app_name (str, optional): application name
        maxBytes (int, optional): maximum bytes threshold for log rollover
        backupCount (int, optional): maximum number of log rotations to keep
        config (dict, optional): configuration overrides

    Returns:
        flask.Flask: flask instance
    """
    flask_app_instance.jinja_env.trim_blocks = True
    flask_app_instance.jinja_env.lstrip_blocks = True
    flask_app_instance.jinja_env.strip_trailing_newlines = True
    flask_app_instance.config["SEND_FILE_MAX_AGE_DEFAULT"] = 24 * 3600 * 365
    flask_app_instance.config["MAX_CONTENT_LENGTH"] = 2 * 1024 * 1024
    flask_app_instance.config["CORS_SUPPORTS_CREDENTIALS"] = True

    if kwargs.get("config"):
        flask_app_instance.config.update(kwargs.get("config"))

    if kwargs.get("jinja_filters"):
        jinja_filters = kwargs.get("jinja_filters")

        for (filter_key, filter_func) in six.iteritems(jinja_filters):
            flask_app_instance.jinja_env.filters[filter_key] = filter_func

    CORS(flask_app_instance)
    Compress(flask_app_instance)

    if kwargs.get("app_name"):
        log_kwargs = dict(
            maxBytes=kwargs.get("maxBytes", ROTATING_LOG_MAX_BYTES),
            backupCount=kwargs.get("backupCount", ROTATING_LOG_BACKUP_COUNT),
        )
        rotating_app_log(
            flask_app_instance, kwargs.get("app_name"), **log_kwargs
        )

    return flask_app_instance


def request_wants_mimetype(mtb, other="text/html"):
    best = request.accept_mimetypes.best_match([mtb, other])
    return (
        best == mtb
        and request.accept_mimetypes[best] > request.accept_mimetypes[other]
    )


def request_wants_json():
    return request_wants_mimetype("application/json")


def generate_expires_header(expires=False):
    """
    Generate HTTP expiration header.

    Args:
        expires: expiration in seconds or False for *imediately / no caching*

    Returns:
        dict: key/value pairs defining HTTP expiration information
    """
    headers = {}

    if expires is False:
        headers["Cache-Control"] = (
            "no-store, no-cache, must-revalidate, "
            "post-check=0, pre-check=0, max-age=0"
        )
        headers["Expires"] = "-1"
    else:
        now = datetime.datetime.now()
        expires_time = now + datetime.timedelta(seconds=expires)
        headers["Cache-Control"] = "public"
        headers["Expires"] = format_date_time(
            time.mktime(expires_time.timetuple())
        )

    return headers



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['coshed_flask']

package_data = \
{'': ['*']}

install_requires = \
['Flask-Compress>=1.10.1,<2.0.0',
 'Flask-Cors>=3.0.10,<4.0.0',
 'Flask-HTTPAuth>=4.5.0,<5.0.0',
 'Flask>=2.0.2,<3.0.0',
 'Werkzeug>=2.0.3,<3.0.0']

setup_kwargs = {
    'name': 'coshed-flask',
    'version': '0.16.0',
    'description': 'flask helper for lazy developer(s)',
    'long_description': None,
    'author': 'doubleO8',
    'author_email': 'wb008@hdm-stuttgart.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<3.10',
}


setup(**setup_kwargs)
